" ---------------------
" BBCODE SYNTAX FOR VIM
" ---------------------
" Original version from Georg Dahn (2008)
" Modified by PowerGod (2016)
"
" * Ver. 1.0.1 BETA
"
" - Added matchit.vim support
" - Removed highlight of tags inside [code] blocks and fixed relative tag colors
" - Added ignorecase for tags (instead of using the global setting)
" - Added sync from start (could be slower for huge files, but without it many scripts can be wrongly highlighted) 
" - Fixed highlight of other tags inside [LIST] (the [*] tag is higlighted in a different way for now)
" - Many tags now ignore spaces 
"
" TODO:
" - Highlight closed tags without an opening match
" - Ignore spaces in all the tags definitions
" - Try a better solution with [code] (because [code] is still highlighted even inside a [code] text...) 
" - Add [code] tag inside bold/italic/underlined text
"
if version < 600
    syntax clear
elseif exists("b:current_syntax")
    finish
endif

" Global case insensitivity, but it's preferably to set it for every tag
"syn case ignore
syn spell toplevel

syn match bbcodeItem contained "\[\s*[-a-zA-Z0-9]\+"hs=s+1 contains=@NoSpell
syn match bbcodeItem contained "\[/\s*[-a-zA-Z0-9]\+"hs=s+2 contains=@NoSpell
" Removed because it was messing with list items
"syn match bbcodeItem contained "\[\s*\*\s*\]"hs=s+1,he=e-1 contains=@NoSpell
syn match bbcodeArgument contained "\s[-a-zA-Z0-9]\+\s*="ms=s+1,me=e-1 contains=@NoSpell
syn region bbcodeValue contained start="\"" end="\"" contains=@NoSpell
syn region bbcodeValue contained start="'" end="'" contains=@NoSpell
syn match bbcodeValue contained "=[\t ]*[^'" \t\]][^ \t\]]*"hs=s+1 contains=@NoSpell
syn region bbcodeTag start="\[/\{0,1}" end="\]" contains=@NoSpell,bbcodeItem,bbcodeArgument,bbcodeValue
" Required for correct highlight of [code] tag
syn region bbcodeCodeTag start="\[\s*\ccode" end="\]" contains=@NoSpell,bbcodeItem,bbcodeArgument,bbcodeValue

syn region bbcodeBold start="\[\cb\]" end="\[/\cb\]"me=e-4 contains=bbcodeTag,bbcodeBoldItalic,bbcodeBoldUnderline,bbcodeCode
syn region bbcodeBoldItalic contained start="\[\ci\]" end="\[/\ci\]"me=e-4 contains=bbcodeTag,bbcodeBoldItalicUnderline
syn region bbcodeBoldItalicUnderline contained start="\[\cu\]" end="\[/\cu\]"me=e-4 contains=bbcodeTag
syn region bbcodeBoldUnderline contained start="\[\cu\]" end="\[/\cu\]"me=e-4 contains=bbcodeTag,bbcodeBoldUnderlineItalic
syn region bbcodeBoldUnderlineItalic contained start="\[\ci\]" end="\[/\ci\]"me=e-4 contains=bbcodeTag

syn region bbcodeItalic start="\[\ci\]" end="\[/\ci\]"me=e-4 contains=bbcodeTag,bbcodeItalicBold,bbcodeItalicUnderline
syn region bbcodeItalicBold contained start="\[\cb\]" end="\[/\cb\]"me=e-4 contains=bbcodeTag,bbcodeItalicBoldUnderline
syn region bbcodeItalicBoldUnderline contained start="\[\cu\]" end="\[/\cu\]"me=e-4 contains=bbcodeTag
syn region bbcodeItalicUnderline contained start="\[\cu\]" end="\[/\cu\]"me=e-4 contains=bbcodeTag,bbcodeItalicUnderlineBold
syn region bbcodeItalicUnderlineBold contained start="\[\cb\]" end="\[/\cb\]"me=e-4 contains=bbcodeTag

syn region bbcodeUnderline start="\[\cu\]" end="\[/\cu\]"me=e-4 contains=bbcodeTag,bbcodeUnderlineBold,bbcodeUnderlineItalic
syn region bbcodeUnderlineBold contained start="\[\cb\]" end="\[/\cb\]"me=e-4 contains=bbcodeTag,bbcodeUnderlineBoldItalic
syn region bbcodeUnderlineBoldItalic contained start="\[\ci\]" end="\[/\ci\]"me=e-4 contains=bbcodeTag
syn region bbcodeUnderlineItalic contained start="\[\ci\]" end="\[/\ci\]"me=e-4 contains=bbcodeTag,bbcodeUnderlineItalicBold
syn region bbcodeUnderlineItalicBold contained start="\[\cb\]" end="\[/\cb\]"me=e-4 contains=bbcodeTag

syn region bbcodeUrl start="\[\curl\s*[=\]]" end="\[/\curl\]"me=e-6 contains=@NoSpell,bbcodeTag

" syn region bbcodeCode matchgroup=Delimiter start="/\[code\(=.\{-}\)\?\]/" end="/\[\/code\]/"
"syn region bbcodeCode matchgroup=Delimiter start="\[code\(=.\{-}\)\?\]"he=e+6 end="\[\/code\]"me=e-7
"syn region bbcodeCode matchgroup=Statement start="\[\ccode\(=.\{-}\)\?\]"he=e+6 end="\[/\ccode\]"me=e-7 contains=@NoSpell,bbcodeCodeTag
"syn region bbcodeCode matchgroup=Statement start="\[\ccode\(=.\{-}\)\?\]" end="\[/\ccode\]"me=e-7 contains=@NoSpell,bbcodeCodeTag
syn region bbcodeCode start="\[\s*\ccode\(\s*=\s*.\{-}\)\?\]" end="\[\s*/\s*\ccode\s*\]"me=e-7 contains=@NoSpell,bbcodeCodeTag

" Definitions for matchit.vim (to jump from starting to closing tags) 
" (using % jumps from [LIST], [*] and [/LIST], else from the starting tag to the closing tag)
"with \{-} does not find the starting tag, so I must use .*
let b:match_words = '\[\cLIST\(=.*\)\?\]:\[\*\]:\[/\cLIST\],\[[^/]\{-}\]:\[/.\{-}\]'
"let b:match_words = '\[\cLIST\(=\{-}\)\?\]:\[\*\]:\[/\cLIST\],\[[^/]\{-}\]:\[/.\{-}\]'

" CODE TO DO SOME TRIALS TO CATCH EXCEDING FINAL TAGS
"catch errors caused by wrong parenthesis
"syn region vfpParen transparent start='(' end=')' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr ")"
"syn region vfpParen transparent start='\[[^/]\{-}\]' end='\[/.\{-}\]' contains=ALLBUT,vfpParenErr
"syn region vfpParen transparent start='\[\([^/]\{-}\)\]' end='\[\/\1\]' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr "\[/.\{-}\]"
"syn region vfpParen transparent start='\[\([^/]\{-}\)\]' end='\[/\1\]' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr "[/.*]"
"syn region vfpParen transparent start='\[' end='\]' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr "\]"

" Define the default highlighting.
" For version 5.7 and earlier: only when not done already
" For version 5.8 and later: only when an item doesn't have highlighting yet
if version >= 508 || !exists("did_vfp_syn_inits")
	if version < 508
		let did_vfp_syn_inits = 1
		command -nargs=+ HiLink hi link <args>
	else
		command -nargs=+ HiLink hi def link <args>
	endif

	HiLink bbcodeTag Identifier
	HiLink bbcodeCodeTag Identifier
	HiLink bbcodeItem Statement
	HiLink bbcodeArgument Type
	HiLink bbcodeValue Constant
	HiLink bbcodeUrl Underlined
	HiLink bbcodeCode PreProc
	HiLink vfpParenErr Error

	HiLink bbcodeBoldUnderlineItalic bbcodeBoldItalicUnderline
	HiLink bbcodeItalicBold bbcodeBoldItalic
	HiLink bbcodeItalicBoldUnderline bbcodeBoldItalicUnderline
	HiLink bbcodeItalicUnderlineBold bbcodeBoldItalicUnderline
	HiLink bbcodeUnderlineBold bbcodeBoldUnderline
	HiLink bbcodeUnderlineBoldItalic bbcodeBoldItalicUnderline
	HiLink bbcodeUnderlineItalic bbcodeItalicUnderline
	HiLink bbcodeUnderlineItalicBold bbcodeBoldItalicUnderline

	delcommand HiLink
endif

hi def bbcodeBold term=bold cterm=bold gui=bold
hi def bbcodeBoldItalic term=bold,italic cterm=bold,italic gui=bold,italic
hi def bbcodeBoldItalicUnderline term=bold,italic,underline cterm=bold,italic,underline gui=bold,italic,underline
hi def bbcodeBoldUnderline term=bold,underline cterm=bold,underline gui=bold,underline
hi def bbcodeItalic term=italic cterm=italic gui=italic
hi def bbcodeItalicUnderline term=italic,underline cterm=italic,underline gui=italic,underline
hi def bbcodeUnderline term=underline cterm=underline gui=underline

" This will make more precise the syntax check
" but could be commented out if it becomes too slow
syntax sync fromstart

let b:current_syntax = "bbcode"
